#ifndef __M200_PROTOCOL_HEADER__
#define __M200_PROTOCOL_HEADER__

/**
 * This is a C language header, defining most of the symbols
 * proposed in the PC Documation M200 C Serial Communication Protocol,
 * as of version 1.00
 *
 * $Id: pc-uc-protocol.h 59 2010-01-22 18:44:34Z sven-win $
 * This file is part of the Punched Paper Project - Punch Card Device Drivers
 * Copyright (C) 2009  Sven Koeppel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see
 * <http://www.gnu.org/licenses/>.
 *
 **/

/* Grundlegende Funktionen */
#define M200_digit2char( digit ) ((char)( digit + '0' - 1 ))

/* Kompatibilitaet */

/* Most important entries*/
#define M200_PROTOCOL_VERSION_MAJOR       0
#define M200_PROTOCOL_VERSION_MINOR       5

/* Comparisation and output routines */
#define M200_PROTOCOL_IS_COMPATIBLE(major, minor) ((major) == M200_PROTOCOL_VERSION_MAJOR && (minor) >= M200_PROTOCOL_VERSION_MINOR)

/* Just for output*/
#define M200_PROTOCOL_VERSION_REVISION    "$Revision: 59 $" 
 
/* Client request codes */
 
#define M200_CLIENT_HELP             'h'
#define M200_CLIENT_PING             '0'
#define M200_CLIENT_START            '1'
#define M200_CLIENT_STOP             '2'
#define M200_CLIENT_HEX              '3'
#define M200_CLIENT_DEBUG            '4'
#define M200_CLIENT_BINARY           '5'
#define M200_CLIENT_RESET            '6'
#define M200_CLIENT_VERBOSE          '7'
 
/* Server response codes */
#define M200_SERVER_PONG             "100"
#define M200_SERVER_HELP             "110"
#define M200_SERVER_STARTUP          "200"
#define M200_SERVER_ACK_PREFIX       "20"
#define M200_SERVER_ACK_START        "201"
#define M200_SERVER_ACK_STOP         "202"
#define M200_SERVER_ACK_HEX          "203"
#define M200_SERVER_ACK_DEBUG        "204"
#define M200_SERVER_ACK_BINARY       "205"
#define M200_SERVER_ACK_RESET        "206"
#define M200_SERVER_ACK_VERBOSE      "207"

/* Signal codes */
#define M200_SERVER_SIGNAL_PREFIX    "3"
#define M200_SERVER_SIGNAL_ERROR     "0"
#define M200_SERVER_SIGNAL_READY     "1"
#define M200_SERVER_SIGNAL_BUSY      "2"
#define M200_SERVER_SIGNAL_HCK       "3"
#define M200_SERVER_SIGNAL_MOCK      "4"
#define M200_SERVER_SIGNAL_RISE      "1"
#define M200_SERVER_SIGNAL_FALL      "0"
// can be composed by preprocessor, like in
// puts(M200_SERVER_SIGNAL_PREFIX M200_SERVER_SIGNAL_READY M200_SERVER_SIGNAL_RISE " Ready rising);

#define M200_SERVER_ILLEGAL_COMMAND  "500"
#define M200_SERVER_ILLEGAL_FOOBAR   "501"
#define M200_SERVER_ILLEGAL_FORMAT   "502"
#

#define M200_SERVER_HEX_PREFIX       "6"
#define M200_SERVER_DEBUG_PREFIX     "7"
#define M200_SERVER_BINARY_PREFIX    "8"
#define M200_SERVER_BINARY           "800" 
#define M200_SERVER_BINARY_FINISHED  "808"

#define M200_SERVER_FATAL_PREFIX      "9"
#define M200_SERVER_FATAL_CARD        "900"
#define M200_SERVER_FATAL_PRINT_BUFFER "905"

 

#endif /* __M200_PROTOCOL_HEADER__ */
