#ifndef PUNCH_CARD_EDITOR_MAINWINDOW_H
#define PUNCH_CARD_EDITOR_MAINWINDOW_H

#include <QMainWindow>
#include <QAction>
#include <QUndoView>
#include <QMenu>
#include <QMenuBar>
#include <QStatusBar>
#include <QToolBar>
#include <QPointer>
#include <QFile>

namespace QPunchCard {
namespace App {
	class MainWindow;
};
};

#include "qpunchcard/card.h"
#include "qpunchcard/deck.h"
#include "deckviewer/navigatordock.h"
#include "deckviewer/cardeditor.h"
#include "text/editordock.h"
#include "driver/driver.h"

namespace QPunchCard {
namespace App {

class MainWindow : public QMainWindow {
	Q_OBJECT
	DeckIndex current_index;

public:
	// gerade offenes *Dokument*
	QPointer< Deck > deck;

	QPointer<Navigator::Dock> navigator;
	QPointer<CardEditor> graphical_editor;
	QList< QPointer<QPunchCard::Text::EditorDock> > text_editors; // eigentlich nur fuer exportText()
	QList< QPointer<QPunchCard::Device::Dock> > device_docks;

	MainWindow();

	bool hasFileOpened() const { return deck; }
	QPointer< Deck > getDeck() const { return deck; }
	DeckIndex getCurrentIndex() const { return current_index; }

	/// speichert die Datei wenn noetig, auf Benutzeranfrage
	/// @returns ob weitergemacht werden darf (wenn nicht, will Benutzer
	/// an seiner Datei weiterarbeiten)
	bool maybeSave();

	// Low-Level-Funktionen ohne GUI-Interaktion
	void loadDeck(const QString& filename);
	void closeDeck();


public slots:
	void newFile();
	void openFile();
	/// Speichert Datei. Wenn kein Dateiname, ruft saveFileAs auf.
	/// @returns True bei erfolgreichem Speichern, sonst false.
	bool saveFile();
	/// Oeffnet Speichern unter-Dialog
	/// @returns true, wenn erfolgreich gespeichert
	bool saveFileAs();
	void exportText();
	//void exportPicture();
	void closeFile();
	void help();
	void quit();
	void about();
	void newTextEditor();

	void newCard();
	void moveCardForwards();
	void moveCardBackwards();

	// durchreichen an alle Kindobjekte
	void nextCard();
	void prevCard();
	void setCard(DeckIndex i);

signals:
	void cardSelected(DeckIndex i);

	// broadcasting from deck
	void contentsChanged(DeckIndex lower_index, DeckIndex upper_index);

	void fileOpened(bool opened);

private slots:
	void window_update_on_file_state(bool opened);
	void actions_update_on_file_state(bool opened);

private:
	void createDevices();
	void createGraphicalEditor();
	void createActions();
	void createMenus();
	void createDynamicMenus();
	void createToolBars();
	void createDynamicToolBars();
	void createStatusBar();
	void createDockWindows();

	void notifyFileOpened();

	// top level menus
	QMenu* file_menu;
	QMenu* edit_menu;
	QMenu* view_menu;
	QMenu* devices_menu;
	QMenu* help_menu;

	// sub menus
	QMenu* toolbars_view_menu;
	QMenu* docks_view_menu;

	QToolBar* navi_bar;
	QToolBar* main_bar;

	QPointer<QUndoView> undo_view;

	// FILE Actions
	QAction
		* new_file_action,
		* open_file_action,
		* save_file_action,
		* save_file_as_action,
		* export_text_action,
		* export_picture_action,
		* close_action,
		* help_action,
		* about_action;

	// EDIT Actions
	QAction
		* undo_action,
		* redo_action,
		* undo_view_action;


	QAction* new_text_editor_action;

	// Naviactions
	QAction* next_card_action;
	QAction* prev_card_action;
	QAction* new_card_action;
	QAction* move_forwards_action;
	QAction* move_backwards_action;

protected:
	void closeEvent(QCloseEvent* event);
};

}; // namespace App
}; // namespace QPunchCard

#endif // PUNCH_CARD_EDITOR_MAINWINDOW_H
