#include "mainwindow.h"
#include "qpunchcard/card.h"
#include "qpunchcard/widget.h"
#include "deckviewer/navigatormodel.h"

#include <QApplication>
#include <QLabel>
#include <QtDebug>
#include <QListView>
#include <QDockWidget>
#include <QHBoxLayout>
#include <QPushButton>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>

using namespace QPunchCard;
using namespace App;


void MainWindow::window_update_on_file_state(bool opened) {
	// * Window Title setzen
	// * contentsChanged-Signal broadcasten
	// * Menus und Actions erstellen
	if(opened) {
		Q_ASSERT(deck);

		// titel setzen
		if(deck->hasFile()) {
			setWindowFilePath(QFileInfo(deck->getFileName()).filePath());
		}
		setWindowTitle(tr("%1[*] - %2").arg(
			deck->hasFile() ? tr("New Card Deck") : deck->getFileName()
			).arg(tr("Punch Card Editor")));
		// Modified-Stern ([*]) verbinden
		connect(deck, SIGNAL(modified(bool)), this, SLOT(setWindowModified(bool)));

		// Broadcasting von "Edited" verbinden
		connect(deck, SIGNAL(contentsChanged(DeckIndex, DeckIndex)), this, SIGNAL(contentsChanged(DeckIndex,DeckIndex)));
	} else {
		setWindowTitle(tr("Punch Card Editor"));
		// passiert folgendes nicht sowieso automatisch?
		disconnect(this, SLOT(setWindowModified(bool)));
		// urhm, den broadcaster... nicht die slots freisetzen, waere bloed.
	}
}

void MainWindow::actions_update_on_file_state(bool opened) {
	// 1#
	if(opened) {
		// Actions erstellen
		undo_action = deck->createUndoAction();
		redo_action = deck->createRedoAction();

		// Undo view
		QDockWidget* undo_view_dock = new QDockWidget(tr("Editing History"), this);
		this->addDockWidget(Qt::RightDockWidgetArea, undo_view_dock);
		undo_view_dock->setVisible(false);
		undo_view_action = undo_view_dock->toggleViewAction();
		undo_view = new QUndoView(deck->getUndoStack(), undo_view_dock);
		undo_view_dock->setWidget(undo_view);

		// Signale verbinden
		connect(deck, SIGNAL(modified(bool)), save_file_action, SLOT(setEnabled(bool)));
		connect(deck, SIGNAL(modified(bool)), save_file_as_action, SLOT(setEnabled(bool)));
		connect(export_picture_action, SIGNAL(triggered()), graphical_editor, SLOT(exportPictureDialog()));

		// dynamische Menues und Toolbars erstellen
		createDynamicMenus();
		createDynamicToolBars();
	} else {
		// dieser Status existiert gar nicht und so!11 (oh man, aber echt)
		qDebug("Calling bad state in actions_update_on_file_state");
	}
}

void MainWindow::createActions() {
	//----------------------- FILE ------------------------------
	// new File: Always possible, open new Window with empty file
	new_file_action = new QAction(QIcon(":/icons/document-new.png"), tr("&New"), this);
	new_file_action->setShortcuts(QKeySequence::New);
	new_file_action->setStatusTip(tr("Open a new window with an empty card deck"));
	connect(new_file_action, SIGNAL(triggered()), this, SLOT(newFile()));

	// open File: Always possible, open file in current window
	open_file_action = new QAction(QIcon(":/icons/document-open.png"), tr("&Open File..."), this);
	open_file_action->setShortcuts(QKeySequence::Open);
	open_file_action->setStatusTip(tr("Open existing card file (any card deck file or text file)"));
	connect(open_file_action, SIGNAL(triggered()), this, SLOT(openFile()));

	// save File: Only possible if current deck can be saved
	save_file_action = new QAction(QIcon(":/icons/document-save.png"), tr("&Save File"), this);
	save_file_action->setShortcuts(QKeySequence::Save);
	save_file_action->setStatusTip(tr("Save the current file as card deck"));
	connect(save_file_action, SIGNAL(triggered()), this, SLOT(saveFile()));
	// connection to deck will be etabilshed when deck is loaded

	// save File as: Only possible if current deck can be saved
	save_file_as_action = new QAction(QIcon(":/icons/document-save-as.png"), tr("&Save File as..."), this);
	save_file_as_action->setShortcuts(QKeySequence::SaveAs);
	save_file_as_action->setStatusTip(tr("Save the current card deck as another file"));
	connect(save_file_as_action, SIGNAL(triggered()), this, SLOT(saveFileAs()));
	// connection to deck will be etabilshed when deck is loaded

	// export Text: Always possible with present deck, deck always present
	export_text_action = new QAction(tr("&Export Text..."), this);
	export_text_action->setStatusTip(tr("Save the decoded plaintext of the current card deck"));
	connect(export_text_action, SIGNAL(triggered()), this, SLOT(exportText()));

	// export Picture: Always possible with present deck, deck always present
	export_picture_action = new QAction(tr("&Export Picture of Card..."), this);
	export_picture_action->setStatusTip(tr("Save the visualisation of one or more cards as a picture"));
	// connect to cardeditor will be etabilshed when deck is loaded

	// close Window: Always possible. Will close Window and if no more windows left => quitting.
	close_action = new QAction(QIcon(":/icons/system-log-out.png"), tr("&Quit"), this);
	close_action->setShortcuts(QKeySequence::Close);
	close_action->setStatusTip(tr("Close current file while staying in the application"));
	connect(close_action, SIGNAL(triggered()), this, SLOT(close()));

	// Display Help: Always possible.
	help_action = new QAction(tr("&Help"), this);
	help_action->setShortcuts(QKeySequence::HelpContents);
	help_action->setStatusTip(tr("Get Help about the usage of this program"));
	connect(help_action, SIGNAL(triggered()), this, SLOT(help()));

	// Display about box: Always possible
	about_action = new QAction(tr("&About"), this);
	about_action->setStatusTip(tr("Display some information about the author and program version"));
	connect(about_action, SIGNAL(triggered()), this, SLOT(about()));

	//----------------------- EDIT ------------------------------


	// Open new Text Editor: Always possible with deck present, deck always present
	new_text_editor_action = new QAction(tr("Open new &Text editor"), this);
	new_text_editor_action->setStatusTip(tr("Display another text editor for editing the dock using a card code"));
	connect(new_text_editor_action, SIGNAL(triggered()), this, SLOT(newTextEditor()));

	next_card_action = new QAction(QIcon(":/icons/go-down.png"), tr("&Next Card"), this);
	next_card_action->setStatusTip(tr("Go to next Card"));
	connect(next_card_action, SIGNAL(triggered()), this, SLOT(nextCard()));

	prev_card_action = new QAction(QIcon(":/icons/go-up.png"), tr("&Previous Card"), this);
	prev_card_action->setStatusTip(tr("Go to previous Card"));
	connect(prev_card_action, SIGNAL(triggered()), this, SLOT(prevCard()));

	new_card_action = new QAction(QIcon(":/icons/list-add.png"), tr("Insert &new Card"), this);
	new_card_action->setStatusTip(tr("Insert new blanco card after current card"));
	connect(new_card_action, SIGNAL(triggered()), this, SLOT(newCard()));

	move_forwards_action = new QAction(QIcon(":/icons/format-indent-more.png"), tr("Move forwards"), this);
	move_forwards_action->setStatusTip(tr("Move current card forwards"));
	connect(move_forwards_action, SIGNAL(triggered()), this, SLOT(moveCardForwards()));

	move_backwards_action = new QAction(QIcon(":/icons/format-indent-less.png"), tr("Move backwards"), this);
	move_backwards_action->setStatusTip(tr("Move current card backwards"));
	connect(move_backwards_action, SIGNAL(triggered()), this, SLOT(moveCardBackwards()));

	// mehr Signale erst beim oeffnen/schliessen einer Datei
}

void MainWindow::createMenus() {
	file_menu = menuBar()->addMenu(tr("&File"));
	file_menu->addAction(new_file_action);
	file_menu->addSeparator();
	file_menu->addAction(open_file_action);
	file_menu->addAction(save_file_action);
	file_menu->addAction(save_file_as_action);
	file_menu->addSeparator();
	file_menu->addAction(export_text_action);
	file_menu->addAction(export_picture_action);
	file_menu->addSeparator();
	file_menu->addAction(close_action);

	// Inhalte von Edit und View erst bei actions_update_on_file_state zugeordnet
	edit_menu = menuBar()->addMenu(tr("&Edit"));
	view_menu = menuBar()->addMenu(tr("&View"));

	// Submenus
	toolbars_view_menu = new QMenu("&Toolbars", this);
	docks_view_menu = new QMenu("&Docks", this);

	devices_menu = menuBar()->addMenu(tr("&Devices"));
	// hier alle Geraete auflisten, fuer die es Treiber gibt (=> driver, DriverOverview oder so)
	// wird von createDevices() gemacht

	menuBar()->addSeparator(); // for Motif ;-)
	help_menu = menuBar()->addMenu(tr("&Help"));
	help_menu->addAction(help_action);
	help_menu->addSeparator();
	help_menu->addAction(tr("About &Qt"), qApp, SLOT(aboutQt()));
	help_menu->addAction(about_action);
}

void MainWindow::createDynamicMenus() {
	// nur von actions_update_on_file_state aufzurufen.
	edit_menu->clear();
	edit_menu->addAction(undo_action);
	edit_menu->addAction(redo_action);
	edit_menu->addAction(undo_view_action);
	edit_menu->addSeparator();
	// Cut, Copy, Insert => Waere ziemlich kompliziert wegen der vielen Modelle
	edit_menu->addAction(new_card_action);
	edit_menu->addAction(move_backwards_action);
	edit_menu->addAction(move_forwards_action);

	view_menu->clear();
	view_menu->addAction(new_text_editor_action);
	view_menu->addSeparator();
	// hier jetzt actions von der EditorKomponente
	// (Zoom, usw.)
	view_menu->addSeparator();
	view_menu->addMenu(toolbars_view_menu);
	view_menu->addMenu(docks_view_menu);
	// Ansichten:
	// - Neue Textansicht hinzufuegen
	// - Eigenschaften der Lochkarten-Ansicht hinzufuegen
	// - Visualisierungsansicht aendern und so (Qualitaet)
	// Navigation
	// - Vor/Zurueck bei Lochkarte oder Spalte (letzteres doof)


}

void MainWindow::createToolBars() {
	main_bar = addToolBar(tr("Main"));
	toolbars_view_menu->addAction(main_bar->toggleViewAction());

	main_bar->addAction(new_file_action);
	main_bar->addAction(open_file_action);
	main_bar->addSeparator();
	main_bar->addAction(save_file_action);
	main_bar->addAction(save_file_as_action);
	main_bar->addSeparator();
	main_bar->addAction(export_text_action);
	// usw.

	navi_bar = addToolBar(tr("Navigation"));
	toolbars_view_menu->addAction(navi_bar->toggleViewAction());

	navi_bar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
	navi_bar->addAction(prev_card_action);
	navi_bar->addAction(next_card_action);
	navi_bar->addAction(new_card_action);
	navi_bar->addAction(this->move_backwards_action);
	navi_bar->addAction(this->move_forwards_action);
}

void MainWindow::createDynamicToolBars() {
	// hier halt sowas wie undo, redo, usw.
	// eigentlich brauchen wir auch nur eine Toolbar...
}
