#include "navigatordock.h"

using namespace QPunchCard;
using namespace Navigator;

Dock::Dock(App::MainWindow* parent) : QDockWidget("Cards", parent), main(parent) {
	model = new Model(this);
	view = new View(this);
	// mach das model selbst:
	//model->setDock(main->deck);
	// darf die view nicht selbst machen (erst nach model erstellt wird!)
	view->setModel(model);

	setWidget(view);
	setFeatures(QDockWidget::AllDockWidgetFeatures);

	// Signale zusammenpatscheln
	connect(view, SIGNAL(activated(QModelIndex)), this, SLOT(cardSelected_translator(QModelIndex)));
	connect(this, SIGNAL(dockLocationChanged(Qt::DockWidgetArea)), this, SLOT(setFlowOnLocationChange(Qt::DockWidgetArea)));
	connect(main, SIGNAL(contentsChanged(DeckIndex,DeckIndex)), model, SLOT(contentsChanged(DeckIndex,DeckIndex)));

	// Beim Klick auf Karte alle Views aktualisieren
	connect(this, SIGNAL(cardSelected(DeckIndex)), main, SLOT(setCard(DeckIndex)));
	connect(main, SIGNAL(cardSelected(DeckIndex)), this, SLOT(setCard(DeckIndex)));

	// Beim Laden/Entladen eines Docks Model und View an/ausschalten
	connect(main, SIGNAL(fileOpened(bool)), this, SLOT(setVisible(bool)));
	connect(main, SIGNAL(fileOpened(bool)), model, SLOT(setDeck(bool)));
}

void Dock::setFlowOnLocationChange(Qt::DockWidgetArea new_area) {
	// Ausrichtung der Icons auf Basis der Ausrichtung des Docks
	switch(new_area) {
		case Qt::TopDockWidgetArea:
		case Qt::BottomDockWidgetArea:
			view->setFlow(QListView::LeftToRight);
			break;
		case Qt::LeftDockWidgetArea:
		case Qt::RightDockWidgetArea:
		default:
			view->setFlow(QListView::TopToBottom);
			break;
	}
	// in jedem Fall Icon-Groesse neu berechnen (etwas quick & dirty)
	resizeEvent(0);
}

void Dock::setCard(DeckIndex i) {
	if(i.isValid())
		view->scrollTo( model->index(i), QAbstractItemView::PositionAtCenter);
	else
		qDebug("Dock invalid scroll to %d", (int)i);
}

void Dock::resizeEvent(QResizeEvent*) {
	if(view->flow() == QListView::TopToBottom) {
		model->setIconSizeByWidth( view->viewport()->contentsRect().width() - 40 );
		view->setIconSize( model->getIconSize() );
	} else if(view->flow() == QListView::LeftToRight) {
		// bei Hoehe ist halt noch das Label irgendwie dabei -- schwierig zu berechnen
		model->setIconSizeByHeight( view->viewport()->contentsRect().height() - 80);
		view->setIconSize( model->getIconSize() );
	}
}
