#include "client.h"

#include "qpunchcard/card.h"
#include "qpunchcard/jones.h"

#include <QDebug>
#include <QByteArray>
#include <QRegExp>
#include <QMutexLocker>

using namespace QPunchCard;
using namespace Device;
using namespace DocumationM200;

ClientThread::ClientThread(QObject* parent) : QThread(parent) {
	abort = false;
}

ClientThread::~ClientThread() {
	abort = true; // Quick and dirty, no mutex.
	qDebug("Waiting for thread to finish.");
	wait();
}

void ClientThread::setAbort(bool value) {
	this->abort = value;
}


void ClientThread::readLine() {
	QMutexLocker locker(&important_reading);

	// Zeile kriegen
	char buf[200];

	qint64 len = port.readLine(buf, sizeof(buf));
	if(len <= 0)
		return;

	QString text(buf);
	if(text.count() != 0) {
		static QRegExp line_regex("^(\\d{3})\\s*([^\\n\\r]*)\\s*$",
					  Qt::CaseInsensitive, QRegExp::RegExp2); // greed
		if(line_regex.indexIn(text) == -1) {
			qDebug() << "No match for" << text;
			return;
		}
		QStringList matches = line_regex.capturedTexts();
		if(matches.count() != 3) {
			qDebug() << "Strange capturing count:" << text << "matched:" << line_regex.capturedTexts();
			return;
		}

		QString code = matches[1];
		QString comment = matches[2];

		emit recievedResponse(code, comment);

		// schauen, ob eine Karte ankommt
		if(M200_SERVER_BINARY == code) {
			qDebug("Processing card!");
			Card card = processCard();

			qDebug("Well... we got a card. Have fun!");
		}
	}
}

/**
 * Call this function *exactly* after you recieved readLine() with code == M200_SERVER_BINARY.
 **/
Card ClientThread::processCard() {
	static const int len = 120;
	char binary[len];
	int offset = 0;

	// read exactly `len` bytes
	do {
		offset += port.read(binary + offset, len - offset);
		qDebug() << "Read in" << offset << "bytes of" << len << "bytes";
	} while( offset != len );

	// we got it. No go on encoding, running the algorithm from the JonesFileFormat
	qDebug() << "Translating data" << QByteArray(binary, len);
	Card card = JonesFileFormat::jones_bytes_to_card(binary);
	qDebug() << "Read in card:" << card;

	// uh... thats it
	return card;
}

void ClientThread::run() {
	qDebug("Device::DocumationM200 ClientThread starting");

	// Open Port:
	port.setPortName("COM4");
	port.setBaudRate(BAUD38400);
	port.setFlowControl(FLOW_OFF);
	port.setDataBits(DATA_8);
	port.setStopBits(STOP_1);
	port.setParity(PAR_NONE);

	//port.setTextModeEnabled(true);
	// => BUG!!!?! Scheint ein Bug in Qt zu sein:
	//   QIODevice bla;
	//   bla.setTextModeEnabled(true);
	//   bla.open(QIODevice::ReadWrite);
	//   qDebug() << bla.openMode() ===> "Text", should be "ReadOnly|Text|WriteOnly"
	//   !!!

	qDebug() << "Open port: " << port.open(QIODevice::ReadWrite | QIODevice::Text);

	qDebug() << "Open mode:" << port.openMode();

	port.putChar( M200_CLIENT_BINARY );
	//qDebug("Printed a char");

	char last_char;
	forever {
		// das mag Qt nicht...
		//QByteArray data = port.readLine( Q_INT64_C(600) );
		//QString text(data);
		//qDebug() << "Read in [" << data.count()  <<  "]: " << data;

		/*
		last_char = (last_char == M200_CLIENT_START) ? M200_CLIENT_STOP : M200_CLIENT_START;
		port.putChar(last_char);

		port.waitForBytesWritten(1000); // mal lieber, bevor was kaputt geht
		*/

		readLine();

		usleep(50);

		if(abort) {
			qDebug("Returning from thread main loop");
			return;
		}

		// noch eine Variante
		/*
		qDebug("Wait for answer");
		while( port.waitForReadyRead(-1) ) {
		//while( port.bytesAvailable()   ) {
			qDebug("GOT OUTPUT!!!");
			QString text(port.readAll());
			qDebug() << "DATA:" << text;
		}
		msleep(1000);
		if(!port.errorString().isEmpty())
			qDebug() << "ERROR:" << port.errorString();
		*/

		// also auf die harte Tour:
		/*
		char buf[200];
		qint64 len = port.readLine(buf, sizeof(buf));
		if(len != -1) {
			QString text(buf);
			if(text.count() != 0) {
				qDebug() << "Read in [" << text.count() << "]: " << text;
			}
		}
		*/

	} // forever
}

/**
 * This can (and must, since there's no Event Loop in this Thread) be called by other
 * threads directly (Qt::DirectConnection when using Signals and Slots).
 **/
void ClientThread::fire_start() {
	qDebug() << "Firing Startup";
	QMutexLocker just_a_short_printout(&important_reading);

	port.putChar( M200_CLIENT_START );
	port.waitForBytesWritten(1000);
}

void ClientThread::fire_stop() {
	qDebug() << "Firing Stop";
	QMutexLocker just_a_short_printout(&important_reading);

	port.putChar( M200_CLIENT_STOP );
	port.waitForBytesWritten(1000);
}

void ClientThread::fire_ping() {
	qDebug() << "Firing Ping";
	QMutexLocker just_a_short_printout(&important_reading);

	port.putChar( M200_CLIENT_PING );
	port.waitForBytesWritten(1000);
}

void ClientThread::fire_reset() {
	qDebug() << "Firing reset";
	QMutexLocker just_a_short_printout(&important_reading);

	port.putChar(M200_CLIENT_RESET);
	port.waitForBytesWritten(1000);
	qDebug() << "Done.";
}
