#include "driver/documation-m200-client/controller.h"

#include <QLabel>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>

using namespace QPunchCard;
using namespace Device;
using namespace DocumationM200;

Controller::Controller(QWidget* parent) : QWidget(parent) {
	// set up the heap widgets
	worker = new ClientThread(this);
	log = new QTextEdit(this);
	QWidget* button_box = createButtonBar();

	// set up the log
	log->setReadOnly(true);

	// create the global box
	QHBoxLayout* layout = new QHBoxLayout;
	layout->addWidget(button_box);
	layout->addWidget(log);
	setLayout(layout);

	// now going to fire up working thread
	connect(worker, SIGNAL(recievedResponse(QString, QString)), this, SLOT(logResponse(QString,QString)), Qt::QueuedConnection);

	qDebug("Starting worker thread");
	worker->start();
	qDebug("Finished starting thread");
}

QWidget* Controller::createButtonBar() {
	QWidget* box = new QWidget(this);
	QHBoxLayout* layout = new QHBoxLayout(box);

 	start_button = new QPushButton(tr("Start"), box);
	connect(start_button, SIGNAL(clicked()), worker, SLOT(fire_start()));

	stop_button = new QPushButton(tr("Stop"), box);
	connect(stop_button, SIGNAL(clicked()), worker, SLOT(fire_stop()));

	ping_button = new QPushButton(tr("Ping"), box);
	connect(ping_button, SIGNAL(clicked()), worker, SLOT(fire_ping()));

	reset_button = new QPushButton(tr("Reset"), box);
	connect(reset_button, SIGNAL(clicked()), worker, SLOT(fire_reset()));

	QWidget* clear_log = new QPushButton(tr("Clear log"), box);
	connect(clear_log, SIGNAL(clicked()), log, SLOT(clear()));

	quit_button = new QPushButton(tr("Quit Client"), box);
	quit_button->setToolTip("Suspend or restart Client Thread");
	connect(reset_button, SIGNAL(clicked()), this, SLOT(client_toggle_run()));

	box->setLayout(layout);
	return box;
}

void Controller::client_toggle_run() {
	// when user clicks quit_button
	bool on = worker->isRunning();

	if(on) {
		worker->setAbort(true);
	} else {
		// fire up client
		worker->start();
	}

	start_button->setEnabled(on);
	stop_button->setEnabled(on);
	ping_button->setEnabled(on);
	reset_button->setEnabled(on);
	quit_button->setText(on ? tr("Quit Client") : tr("Restart Client"));
}

void Controller::logResponse(QString code, QString comment) {
	log->moveCursor(QTextCursor::End);
	log->insertHtml("<br><b><font color=red>"+code+"</font></b> "+comment);
}
