#include "driver.h"

#include "driver/documation-m200-client/controller.h"

#include <QLabel>

using namespace QPunchCard;
using namespace Device;

Dock::Dock(QString driver_name, App::MainWindow* parent) : QDockWidget(parent), main(parent), driver_name(driver_name) {
	setWindowTitle( tr("%1 Device Driver").arg(driver_name) );
	connect(this, SIGNAL(visibilityChanged(bool)), this, SLOT(visibilityHook(bool)));
}

void Dock::visibilityHook(bool visible) {
	// Singleton behaviour
	if(!visible || controller)
		return;

	qDebug() << "Firing up device driver " << driver_name;
	// first making visible. Create controller widget.
	controller = DriverFactory::createController(driver_name, this);
	if(!controller) {
		// Bad name! So create an error message
		controller = new QLabel(tr("<b>Bad controller:</b> <i>%1</i> doesn't exist").arg(driver_name), this);
	}
	setWidget(controller);
	controller->setVisible(true);
}

QList<QString> DriverFactory::availableDrivers() {
	QList<QString> ret;
	ret << "Test" << "Documation M200 Card Reader";
	return ret;
}

QWidget* DriverFactory::createController(const QString& name, QWidget* parent) {
	if(name == "Test") {
		return new QLabel("This is the Test driver :-)", parent);
	} else if(name == "Documation M200 Card Reader") {
		return new DocumationM200::Controller(parent);
	} else {
		qDebug() << "DriverFactory: Bad driver: " << name;
		return 0;
	}
}

Dock* DriverFactory::createDock(const QString& name, App::MainWindow* window) {
	// yay, this is senseless...
	return new Dock(name, window);
}

