#include "deck.h"

using namespace QPunchCard;

Deck::Deck(QObject* parent) : QObject(parent), undo_stack(this) {
	init();
}

Deck::Deck(FileFormat format, const QString& filename, QObject* parent) :
		QObject(parent), undo_stack(this), format(format), filename(filename) {
	init();
	read();
}

void Deck::init() {
	// undo ist jetzt direktes member, nicht mher noetig
	//undo = new QUndoStack(this);
}

bool Deck::save() {
	return hasFile() ? save(filename) : false;
}

bool Deck::save(const QString& filename) {
	return hasFormat() ? save(format, filename) : false;
}

bool Deck::save(FileFormat format, const QString& filename) {
	QFile file(filename);
	return save(format, file);
}

bool Deck::save(FileFormat format, QFile& file) {
	if( format.write(file, this) ) {
		// save filename and format
		this->format = format;
		this->filename = file.fileName();
		return true;
	} else {
		qDebug() << "Deck saving failed. Format: " << format.getName();
		return false;
	}
}

bool Deck::read() {
	return hasFile() ? read(filename) : false;
}

bool Deck::read(const QString& filename) {
	return hasFormat() ? save(format, filename) : false;
}

bool Deck::read(FileFormat format, const QString &filename) {
	QFile file(filename);
	return read(format, file);
}

bool Deck::read(FileFormat format, QFile& file) {
	if( format.read(file, this) ) {
		this->format = format;
		this->filename = file.fileName();
		return true;
	} else {
		return false;
	}
}

void Deck::run(DeckUndoCommand* command, bool call_redo) {
	this->undo_stack.push(command);
	if(call_redo)
		command->redo();
}

bool Deck::isValid(int i) {
	return createIndex(i).isValid();
}

DeckIndex Deck::createIndex(int i) {
	return DeckIndex(this, i);
}

void Deck::emitChanged(DeckIndex lower, DeckIndex upper) {
	emit contentsChanged(lower, upper);
}

bool Deck::insert(DeckIndex after) {
	// TODO: Undo Redo Framework
	if(!after.assertDeck(this))
		return false;
	DeckIndex new_index = createIndex(after + 1);
	qDebug("Inserting card at %i", (int)new_index.normalized());
	cards.insert( new_index.normalized(), Card() );
	qDebug() << "Deck is now:" << *this;
	emit contentsChanged(after, createIndex(count()) );
	return true;
}

bool Deck::insertTimes(DeckIndex after, int times) {
	// ja, das ist bloed, aber reicht erst mal:
	for(;times>0;times--) insert(++after);
	return true;
}

void Deck::append(Card card) {
	// man ist das bloed:
	cards.push_back(card);
}


bool Deck::move(DeckIndex /*before*/, DeckIndex /*after*/) {
	// asdasdas
	return true;
	// emit cardChanged
}

bool Deck::erase(DeckIndex from, DeckIndex to) {
	qDebug() << "Deck: Erasing from" << from << " to " << to;
//	main->deck->erase( main->deck->begin() += new_max_blocks+1,
//	   main->deck->begin() += end_block_number+1 );
	return true;
	// emit cardChanged
}




QDebug QPunchCard::operator<<(QDebug dbg, const Deck &c) {
	dbg.nospace() << "QPunchCard::File object with [" << c.count() << "] cards:\n\n";
	for(int i = 0; i < c.size(); i++) {
		dbg.nospace() << "Card no. #" << i << " (" << c.at(i) << ")\n";
		dbg.nospace() << c.at(i);
		dbg.nospace() << '\n';
	}
	return dbg.nospace();
}

QDebug QPunchCard::operator<<(QDebug dbg, const DeckIndex &c) {
	dbg.nospace() << "[QPunchCard::DeckIndex: i=" << c.asInt() << ". Has deck: " << c.hasDeck() << "]";
	return dbg.nospace();
}
