#ifndef FORMAT_H
#define FORMAT_H

#include <QFile>
#include <QList>
#include <QString>

namespace QPunchCard {
	class FileFormat;
	class FileFormatFactory;

	class Deck;
};

#include "qpunchcard/card.h"
// #include "qpunchcard/deck.h" // nach unten verschoben, damit bloeder cc kompiliert

namespace QPunchCard {

class FileFormat {
	QString name;
	friend class FileFormatFactory;
public:
	virtual ~FileFormat() {}
	virtual bool write(QFile& /* target */, const Deck* /* source */) const { qDebug("Default FileFormat write!"); return false; }
	virtual bool read(QFile& /* source */, Deck* /* target */) const { qDebug("Default FileFormat read!"); return false; }
	const QString& getName() const { return name; }
	bool isNull() const { return name.isNull(); }
};

class FileFormatFactory {
public:
	static QList<QString> availableFormats();
	static FileFormat createFormat(const QString& name);
	static QString autoDetectFormat(const QFile& file);
};

/****************************************************************************
  FileFormat (I/O) implementations (see jones.h for the most important one)
 ***************************************************************************/

class PunchCardMarkupLanguageFormat : public FileFormat {
public:
	bool write(QFile& target, const Deck* source) const { return false; }
	bool read(QFile& source, Deck* target) const { return false; }
};

}; // namespace QPunchCard

// hier eingebunden
#include "qpunchcard/deck.h"

#endif // FORMAT_H
