#ifndef JONES_H
#define JONES_H

#include <QBuffer>
#include <QIODevice>

namespace QPunchCard {
	class JonesFileFormat;
};

#include "qpunchcard/format.h"

namespace QPunchCard {

/**
 * FileFormat definition after Douglas W. Jones.
 * This class also implements his core C implementations of writing and reading cards,
 * just suitable for using the algorithms at other classes
 **/
class JonesFileFormat : public FileFormat {
public:
	// low level reading and writing
	static int jones_column_to_integer(const Column& col);
	static Column jones_integer_to_column(int integer);

	// static reading
	static Card jones_bytes_to_card(const char* data);
	static Card jones_bytes_to_card(const char* data, int offset) {
		jones_bytes_to_card(data+offset); }
	static Card jones_bytes_to_card(QByteArray data, int offset = 0);

	// static writing
	static void jones_card_to_bytes(const Card& card, QIODevice& out_device);
	static QByteArray jones_card_to_bytes(const Card& card);

	// high level reading and writing
	bool write(QFile& target, const Deck* source) const;
	bool read(QFile& source, Deck* target) const;
}; // class

}; // namespace

#endif // JONES_H
