#include "editordock.h"

#include <QPainter>
#include <QTextBlock>
#include <QBoxLayout>
#include <QLabel>
#include <QComboBox>
#include <QPushButton>
#include <QFileDialog>
#include <QMessageBox>

using namespace QPunchCard;

Text::EditorDock::EditorDock(App::MainWindow* parent) : QDockWidget(parent), main(parent) {
	// Title grundsaetzlich angeben (wird ueberall angezeigt)
	setWindowTitle(tr("Text Editor"));

	editor = new Text::Editor(main);

	// Signal weiterleiten
	connect(editor, SIGNAL(cursorRowChanged(DeckIndex)), this, SIGNAL(cardSelected(DeckIndex)));
	connect(main, SIGNAL(contentsChanged(DeckIndex,DeckIndex)), editor, SLOT(contentsChanged(DeckIndex,DeckIndex)));

	// urhm... Codebar erstellen
	create_code_bar();
	create_color_bar();

	// Container-Widget erstellen
	QWidget* container = new QWidget(this);
	QVBoxLayout* layout = new QVBoxLayout(container);

	//layout->addWidget(code_bar);
	layout->addWidget(editor);
	layout->addWidget(color_bar);

	setWidget(container);
	setTitleBarWidget(code_bar);
	setFeatures(QDockWidget::AllDockWidgetFeatures);
}

void Text::EditorDock::create_code_bar() {
	code_bar = new QWidget(this);
	QBoxLayout* layout = new QBoxLayout(QBoxLayout::LeftToRight, code_bar);

	// Label fuer Fenster
	layout->addWidget(new QLabel(tr("Card Text"), code_bar));

	// Codec-Auswahlbox
	QComboBox* codec_selection = new QComboBox(code_bar);
	codec_selection->addItems( CodecFactory::availableCodecs() );
	codec_selection->setStatusTip(tr("Use another codec to display the decks contents as ASCII text. This won't change the cards contents"));
	connect(codec_selection, SIGNAL(currentIndexChanged(QString)), this, SLOT(setCodec(QString)));
	layout->addWidget(codec_selection);

	// Button zur Codec-Umwandlung
	QPushButton* text_converter = new QPushButton(tr("Convert text..."), code_bar);
	text_converter->setStatusTip(tr("Convert the current deck (or selection) to another punch card code"));
	connect(text_converter, SIGNAL(clicked()), this, SLOT(showTextConverterDialog()));
	layout->addWidget(text_converter);
}

void Text::EditorDock::create_color_bar() {
	color_bar = new QWidget(this);
	// usw.
}

void Text::EditorDock::setCard(DeckIndex i) {
	editor->highlightRow(i);
}

const Codec* Text::EditorDock::getCodec() const {
	return editor->codec;
}

QString Text::EditorDock::getText() const {
	return editor->toPlainText();
}

void Text::EditorDock::setCodec(QString by_name) {
	// QSharedPointer: Durch Assignment wird das alte Objekt geloescht :-)
	//editor->codec = QSharedPointer<const Codec>( CodecFactory::createCodec(by_name) );
	editor->codec = CodecFactory::createCodec(by_name);
	// jetzt: Text komplett neu auswerten. Todo...
}

bool Text::EditorDock::exportTextAs() {
	// show a funny `export text as` dialog
	QString new_filename = QFileDialog::getSaveFileName(main,
		tr("Export Card Text (%1)").arg(getCodec()->getName()),
		filename.isEmpty() ? QDir::homePath() : QFileInfo(filename).absolutePath(),
		tr("Plain Text File (*.txt);;Any File (*)"));
	if(new_filename.isEmpty())
		return false;

	// save filename and go on exporting normally
	this->filename = new_filename;
	this->exportText();
}

bool Text::EditorDock::exportText() {
	if(this->filename.isEmpty()) {
		return exportTextAs();
	}

	QFile file(this->filename);
	// everything in one line ;-)
	if(!file.open( QIODevice::WriteOnly | QIODevice::Text ) ||
	   (file.write( this->getText().toAscii() ) == -1)  ) {
		QMessageBox::critical(main, tr("Could not export to file"),
				      tr("Plain text export to file %1 failed: %2").arg(filename).arg(file.errorString()));
		return false;
	} else {
		main->statusBar()->showMessage(tr("Successfully exported card text as %1 to %2 ").arg(getCodec()->getName()).arg(filename), 5000);
		return true;
	}
}

void Text::EditorDock::showTextConverterDialog() {
	QMessageBox::information(main, tr("Pending work"), tr("Well... the Text Converter Dialog hasn't been done yet..."));
}
